﻿using System;
using Loki.Engine.Registration;

namespace Loki.Engine
{
    /// <summary>
    /// Common interface for object contexts.
    /// </summary>
    public interface IObjectContext : IDisposable
    {
        /// <summary>
        /// Loads the additional definitions.
        /// </summary>
        /// <param name="P_ObjectDefinitionURI">The object definition URI.</param>
        void LoadContext(params string[] P_ObjectDefinitionURI);

        /// <summary>
        /// Gets an instance of type <typeparamref name="T"/> as defined in the configuration.
        /// </summary>
        /// <typeparam name="T">The object type.</typeparam>
        /// <param name="P_ObjectName">Name of the object.</param>
        /// <returns>The object.</returns>
        T Resolve<T>(string P_ObjectName) where T : class;

        /// <summary>
        /// Gets an instance of type <typeparamref name="T"/> as defined in the configuration.
        /// </summary>
        /// <typeparam name="T">The object type.</typeparam>
        /// <returns>The object.</returns>
        T Resolve<T>() where T : class;

        /// <summary>
        /// Defines the type for creation.
        /// </summary>
        /// <typeparam name="I">Generic type.</typeparam> <typeparam name="T">Concrete
        /// type.</typeparam>
        void Register<T>(ElementRegistration<T> P_Definition) where T : class;

        /// <summary>
        /// Releases the specified object.
        /// </summary>
        /// <param name="P_Object">The object.</param>
        void Release(object P_Object);
    }
}