﻿using System;
using Loki.Data;
using Loki.UI;
using Loki.Utils;

namespace Loki.Engine.Registration
{
    public static class Element
    {
        public static ElementRegistration<TService> For<TService>() where TService : class
        {
            return new ElementRegistration<TService>();
        }

        public static ElementRegistration For(Type P_Type)
        {
            return new ElementRegistration(P_Type);
        }

        public static ElementRegistration<TService> Service<TService, TImplementation>()
            where TService : class
            where TImplementation : class, TService
        {
            return For<TService>().ImplementedBy<TImplementation>().Lifestyle.Singleton;
        }

        public static ElementRegistration<TEntity> Entity<TEntity, TImplementation>()
            where TEntity : class
            where TImplementation : class, TEntity
        {
            return For<TEntity>().ImplementedBy<TImplementation>().Lifestyle.NoTracking;
        }

        public static ElementRegistration<TViewModel> ViewModel<TViewModel>() where TViewModel : class
        {
            return For<TViewModel>().Lifestyle.Transient;
        }

        public static ElementRegistration<IView> View<TView, TViewModel>()
            where TViewModel : class
            where TView : class, IView
        {
            return For<IView>().Named(typeof(TViewModel).FullName).ImplementedBy<TView>().Lifestyle.Transient;
        }

        public static ElementRegistration<TMapper> Mapper<TMapper>(string P_GetAllRequest, string P_GetRequest, string P_DeleteRequest, string P_InsertRequest, string P_UpdateRequest) where TMapper : class, IEntityMapper
        {
            return For<TMapper>().Properties(
              Property<TMapper>.ForKey<string>(x => x.GetRequest).Value(P_GetRequest),
              Property<TMapper>.ForKey<string>(x => x.GetAllRequest).Value(P_GetAllRequest),
              Property<TMapper>.ForKey<string>(x => x.DeleteRequest).Value(P_DeleteRequest),
              Property<TMapper>.ForKey<string>(x => x.InsertRequest).Value(P_InsertRequest),
              Property<TMapper>.ForKey<string>(x => x.UpdateRequest).Value(P_UpdateRequest)
              );
        }

        public static ElementRegistration<object> Mapper(Type P_Mapper, string P_GetAllRequest, string P_GetRequest, string P_DeleteRequest, string P_InsertRequest, string P_UpdateRequest)
        {
            return For(P_Mapper).Properties(
                Property<object>.ForKey(ExpressionHelper.GetProperty<IEntityMapper, string>(x => x.GetRequest)).Value(P_GetRequest),
                Property<object>.ForKey(ExpressionHelper.GetProperty<IEntityMapper, string>(x => x.GetAllRequest)).Value(P_GetAllRequest),
                Property<object>.ForKey(ExpressionHelper.GetProperty<IEntityMapper, string>(x => x.DeleteRequest)).Value(P_DeleteRequest),
                Property<object>.ForKey(ExpressionHelper.GetProperty<IEntityMapper, string>(x => x.InsertRequest)).Value(P_InsertRequest),
                Property<object>.ForKey(ExpressionHelper.GetProperty<IEntityMapper, string>(x => x.UpdateRequest)).Value(P_UpdateRequest)
                );
        }
    }
}