﻿using System;
using System.Collections.Generic;

namespace Loki.Engine.Registration
{
    public class ElementRegistration<TService> where TService : class
    {
        public ElementRegistration()
        {
            Types = new List<Type>();
            Lifestyle = new Livestyle<TService>(this);
            Types.Add(typeof(TService));
        }

        public string Name { get; private set; }

        public IList<Type> Types { get; private set; }

        public Type Implementation { get; private set; }

        public Livestyle<TService> Lifestyle { get; private set; }

        private Dictionary<string, Property<TService>> _ConfiguredProperties = new Dictionary<string, Property<TService>>();

        private Dictionary<string, Property<TService>> _ConfiguredOverrides = new Dictionary<string, Property<TService>>();

        public IEnumerable<Property<TService>> ConfiguredProperties { get { return _ConfiguredProperties.Values; } }

        public IEnumerable<Property<TService>> ConfiguredOverrides { get { return _ConfiguredOverrides.Values; } }

        public ElementRegistration<TService> Named(string P_Name)
        {
            Name = P_Name;
            return this;
        }

        public ElementRegistration<TService> ImplementedBy<TConcrete>() where TConcrete : TService
        {
            Implementation = typeof(TConcrete);
            return this;
        }

        public ElementRegistration<TService> Instance(TService P_Instance)
        {
            return Lifestyle.Instance(P_Instance);
        }

        public ElementRegistration<TService> Properties(params Property<TService>[] P_Properties)
        {
            if (P_Properties != null)
            {
                foreach (var L_Property in P_Properties)
                {
                    _ConfiguredProperties[L_Property.Key.Name] = L_Property;
                }
            }

            return this;
        }

        public ElementRegistration<TService> Overrides(params Property<TService>[] P_Overrides)
        {
            if (P_Overrides != null)
            {
                foreach (var L_Property in P_Overrides)
                {
                    _ConfiguredOverrides[L_Property.Key.Name] = L_Property;
                }
            }

            return this;
        }
    }
}