﻿using System;
using Loki.Utils;

namespace Loki.Engine
{
    public class ObjectModel : LokiObject, IObjectCreator
    {
        public IObjectContext InternalContext
        {
            get;
            set;
        }

        public T Create<T>() where T : class
        {
            return InternalContext.Resolve<T>();
        }

        public T Get<T>() where T : class
        {
            return InternalContext.Resolve<T>();
        }

        public T Create<T>(string P_Name) where T : class
        {
            return InternalContext.Resolve<T>(P_Name);
        }

        public T Get<T>(string P_Name) where T : class
        {
            return InternalContext.Resolve<T>(P_Name);
        }

        #region Load

        public event EventHandler<ContextLoadEventArgs> Load;

        protected virtual void OnLoad(ContextLoadEventArgs e)
        {
            EventHandler<ContextLoadEventArgs> L_Handler = Load;

            if (L_Handler != null)
            {
                L_Handler(this, e);
            }
        }

        #endregion Load

        public void Initialize()
        {
            if (InternalContext == null)
            {
                InternalContext = ToolKit.Engine.CreateContext(Guid.NewGuid().ToString());
            }

            InternalLoad();
            OnLoad(new ContextLoadEventArgs(InternalContext));
        }

        protected virtual void InternalLoad()
        {
        }
    }
}