﻿using Loki.Model;
using Loki.Utils;

namespace Loki.Engine
{
    public abstract class ApplicationService<TModelContext, TDataContext> : LokiService
        where TModelContext : class, IObjectCreator
        where TDataContext : class, IObjectCreator
    {
        #region Model
        /// <summary>
        /// Gets the application model.
        /// </summary>
        /// <value>
        /// The application model.
        /// </value>
        public virtual TModelContext ModelContext
        {
            get { return ToolKit.Engine.GetApplicationModel<TModelContext>(); }
        }

        /// <summary>
        /// Gets the data context.
        /// </summary>
        /// <value>
        /// The data context.
        /// </value>
        public virtual TDataContext DataContext
        {
            get { return ToolKit.Engine.GetDataModel<TDataContext>(); }
        }

        #endregion

        protected EntityContainer<TEntity, TKey> CreateContainer<TEntity, TKey>(ContainerConfiguration<TEntity, TKey> P_Configuration)
            where TEntity : IAtomicEntity<TKey>
        {
            EntityContainer<TEntity, TKey> L_Container = new EntityContainer<TEntity, TKey>(P_Configuration);
            try
            {
                RegisterDisposable(L_Container);
                return L_Container;
            }
            catch
            {
                if (L_Container != null)
                {
                    L_Container.Dispose();
                }

                throw;
            }
        }

        protected EntityLinkContainer<TKey> CreateLinkContainer<TKey>(LinkContainerConfiguration<TKey> P_Configuration)
        {
            EntityLinkContainer<TKey> L_Container = new EntityLinkContainer<TKey>(P_Configuration);
            try
            {
                RegisterDisposable(L_Container);
                return L_Container;
            }
            catch
            {
                if (L_Container != null)
                {
                    L_Container.Dispose();
                }

                throw;
            }
        }
    }
}
