﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Diagnostics;
using System.Linq;
using Loki.Engine;
using Loki.Resources;

namespace Loki.Utils
{
    public class LokiExtensionManager
    {
        private static CompositionContainer _Container;

        private const string DEFAULT_LOGGER = "Log4Net";
        private const string DEFAULT_ENGINE = "Windsor";

        [ImportMany]
        public IEnumerable<Lazy<ILogger, IExtensionType>> Loggers { get; set; }

        public ILogger Logger
        {
            get;
            set;
        }

        [ImportMany]
        public IEnumerable<Lazy<IEngine, IExtensionType>> Engines { get; set; }

        public IEngine Engine
        {
            get;
            set;
        }

        public LokiExtensionManager()
        {
        }

        public void Initialize()
        {
            // An aggregate catalog that combines multiple catalogs
            var L_Catalog = new AggregateCatalog();

            // Adds all the parts found in the same appdomain path
            L_Catalog.Catalogs.Add(new DirectoryCatalog(AppDomain.CurrentDomain.BaseDirectory));

            // Create the CompositionContainer with the parts in the catalog
            _Container = new CompositionContainer(L_Catalog);

            // Fill the imports of this object
            try
            {
                _Container.ComposeParts(this);

                // Assing Logger
                if (Loggers.Count() == 1)
                {
                    Logger = Loggers.First().Value;
                }
                else
                {
                    var L_Extension = Loggers.First(x => x.Metadata.Type == DEFAULT_LOGGER);
                    if (L_Extension != null)
                    {
                        Logger = L_Extension.Value;
                    }
                }

                if (Logger == null)
                {
                    Trace.Fail(ErrorMessages.Utils_ExtensionManager_NoLogger);
                }

                // Assing Engine
                if (Engines.Count() == 1)
                {
                    Engine = Engines.First().Value;
                }
                else
                {
                    var L_Extension = Engines.First(x => x.Metadata.Type == DEFAULT_ENGINE);
                    if (L_Extension != null)
                    {
                        Engine = L_Extension.Value;
                    }
                }

                if (Engine == null)
                {
                    Trace.Fail(ErrorMessages.Utils_ExtensionManager_NoEngine);
                }
            }
            catch (CompositionException compositionException)
            {
                Trace.Fail(compositionException.ToString());
            }
        }
    }
}