﻿using System.ComponentModel;
using Loki.Data;
using Loki.Utils;

namespace Loki.Model
{
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1005:AvoidExcessiveParametersOnGenericTypes")]
    public abstract class AtomicEntity<TData, TKey, TModel, TDataModel> : Entity<TModel, TDataModel>
        where TModel : class, IObjectCreator
        where TDataModel : class, IObjectCreator
        where TData : class, IDataEntity<TKey>
    {
        public TData Data
        {
            get;
            private set;
        }

        #region ID

        private static PropertyChangedEventArgs _IDChangedArgs = ObservableHelper.CreateChangedArgs<AtomicEntity<TData, TKey, TModel, TDataModel>>(x => x.ID);
        private static PropertyChangingEventArgs _IDChangingArgs = ObservableHelper.CreateChangingArgs<AtomicEntity<TData, TKey, TModel, TDataModel>>(x => x.ID);

        public TKey ID
        {
            get
            {
                return Data.PK;
            }

            set
            {
                if (!object.Equals(value, ID))
                {
                    NotifyChanging(_IDChangingArgs);
                    Data.PK = value;
                    NotifyChangedAndDirty(_IDChangedArgs);
                }
            }
        }

        #endregion ID

        protected AtomicEntity()
            : base()
        {
            Data = DataModel.Create<TData>();
            Data.AlreadyPersisted = false;
        }

        protected AtomicEntity(AtomicEntity<TData, TKey, TModel, TDataModel> P_Clone)
            : base(P_Clone)
        {
            BeginInit();
            Data = DataModel.Create<TData>();
            ID = P_Clone.ID;
            Data.AlreadyPersisted = P_Clone.Data.AlreadyPersisted;
            EndInit(!IsChanged);
        }

        protected AtomicEntity(TData P_Clone)
            : base()
        {
            Data = P_Clone;
        }
    }
}