﻿using Loki.Utils;

namespace Loki.Model
{
    /// <summary>
    /// Base class for entities.
    /// </summary>
    public abstract class Entity<TModel, TDataModel> : TrackedObject
        where TModel : class, IObjectCreator
        where TDataModel : class, IObjectCreator
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Entity&lt;TModel,TDataModel&gt;"/> class.
        /// </summary>
        protected Entity()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Entity&lt;TModel,TDataModel&gt;"/> class.
        /// </summary>
        /// <param name="P_Clone">The clone.</param>
        protected Entity(TrackedObject P_Clone)
            : base(P_Clone)
        {
        }

        #endregion Constructors

        #region Model

        /// <summary>
        /// Gets the application model.
        /// </summary>
        /// <value>The application model.</value>
        public TModel ApplicationModel
        {
            get { return ToolKit.Engine.GetApplicationModel<TModel>(); }
        }

        /// <summary>
        /// Gets the data model.
        /// </summary>
        /// <value>The data model.</value>
        public TDataModel DataModel
        {
            get { return ToolKit.Engine.GetDataModel<TDataModel>(); }
        }

        #endregion Model
    }
}