﻿using System;
using Loki.Engine;

namespace Loki.UI
{
    /// <summary>
    /// Common interface for application. Used by bootstrapper.
    /// </summary>
    public interface IApplication
    {
        /// <summary>
        /// Exits the application.
        /// </summary>
        void ApplicationExit();

        /// <summary>
        /// Gets the templating engine.
        /// </summary>
        ITemplatingEngine Templates { get; }

        /// <summary>
        /// Gets the application model.
        /// </summary>
        ObjectModel ApplicationModel { get; }

        /// <summary>
        /// Gets the data model.
        /// </summary>
        ObjectModel DataModel { get; }

        /// <summary>
        /// Gets the UI model.
        /// </summary>
        ObjectModel UIModel { get; }

        /// <summary>
        /// Occurs when an unhandled error occurs.
        /// </summary>
        event EventHandler<ExceptionEventArgs> LastChanceError;

        void Message(string P_Message);

        void Warning(string P_Warning);

        void Error(Exception P_Exception, bool P_Imperative);

        IView MainView { get; set; }
    }
}