﻿using System;
using System.Collections.Generic;

namespace Loki.Utils
{
    /// <summary>
    /// Loki services base class.
    /// </summary>
    public abstract class LokiService : LokiObject, IDisposable
    {
        #region IDisposable

        /// <summary>
        /// Releases all resources used by an instance of the <see cref="LokiService" /> class.
        /// </summary>
        /// <remarks>
        /// This method calls the virtual <see cref="Dispose(bool)" /> method, passing in
        /// <strong>true</strong>, and then suppresses finalization of the instance.
        /// </remarks>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~LokiService()
        {
            Dispose(false);
        }

        /// <summary>
        /// Releases the unmanaged resources used by an instance of the <see cref="LokiService" />
        /// class and optionally releases the managed resources.
        /// </summary>
        /// <param name="P_Disposing">Set to<strong>true</strong> to release both managed and
        /// unmanaged resources; <strong>false</strong> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool P_Disposing)
        {
            foreach (IDisposable L_Disposable in _Disposables)
            {
                L_Disposable.Dispose();
            }

            _Disposables.Clear();
        }

        #endregion IDisposable

        #region Disposable handling

        protected void RegisterDisposable(IDisposable P_Disposable)
        {
            _Disposables.Add(P_Disposable);
        }

        private HashSet<IDisposable> _Disposables = new HashSet<IDisposable>();

        #endregion Disposable handling
    }
}