﻿using System.Linq;
using System.Windows;
using System.Windows.Interactivity;

namespace Loki.UI.Wpf
{
    public abstract class LokiBehaviour<TControl, TBehaviour> : Behavior<TControl>
        where TControl : DependencyObject
        where TBehaviour : Behavior, new()
    {
        public static DependencyProperty IsLinkedProperty =

            DependencyProperty.RegisterAttached("IsLinked", typeof(bool), typeof(LokiBehaviour<TControl, TBehaviour>), new FrameworkPropertyMetadata(false, OnIsLinkedChanged));

        public static bool GetIsEnabled(DependencyObject uie)
        {
            return (bool)uie.GetValue(IsLinkedProperty);
        }

        public static void SetIsEnabled(DependencyObject uie, bool value)
        {
            uie.SetValue(IsLinkedProperty, value);
        }

        private static void OnIsLinkedChanged(DependencyObject dpo, DependencyPropertyChangedEventArgs e)
        {
            UIElement uie = dpo as UIElement;

            if (uie != null)
            {
                var behColl = Interaction.GetBehaviors(uie);

                var existingBehavior = behColl.OfType<TBehaviour>().FirstOrDefault();

                if ((bool)e.NewValue == false && existingBehavior != null)
                {
                    behColl.Remove(existingBehavior);
                }

                else if ((bool)e.NewValue == true && existingBehavior == null)
                {
                    behColl.Add(new TBehaviour());
                }
            }
        }
    }
}