﻿using System;
using System.Linq.Expressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using DevExpress.Xpf.Editors;
using Loki.Utils;

namespace Loki.UI.Wpf
{
    public class ViewControl<TViewModel> : ContentControl, IView, IView<TViewModel> where TViewModel : class, IViewModel
    {
        #region Log

        /// <summary>
        /// Gets the logger.
        /// </summary>
        /// <value>The logger.</value>
        protected ILog Log
        {
            get
            {
                return _LokiAdapter.CommonLog;
            }
        }

        #endregion Log

        private ContentControlAdapter<ViewControl<TViewModel>, TViewModel> _LokiAdapter;

        public ViewControl()
        {
            _LokiAdapter = new ContentControlAdapter<ViewControl<TViewModel>, TViewModel>(this);
            this.Unloaded += ViewControl_Unloaded;
        }

        private void ViewControl_Unloaded(object sender, RoutedEventArgs e)
        {
            OnClean(EventArgs.Empty);
        }

        public TViewModel ViewModel
        {
            get { return DataContext as TViewModel; }
            set { DataContext = value; }
        }

        #region Disposed

        public event EventHandler Disposed;

        protected virtual void OnDisposed(EventArgs e)
        {
            EventHandler L_Handler = Disposed;

            if (L_Handler != null)
            {
                L_Handler(this, e);
            }
        }

        #endregion Disposed

        public Binding AddBinding(Expression<Func<TViewModel, object>> P_Property, DependencyObject P_Target, DependencyProperty P_Targetproperty)
        {
            return _LokiAdapter.AddBinding(P_Property, P_Target, P_Targetproperty);
        }

        public Binding AddValueBinding(Expression<Func<TViewModel, object>> P_Property, BaseEdit P_Control)
        {
            return _LokiAdapter.AddValueBinding(P_Property, P_Control);
        }

        public void Show()
        {
        }

        IViewModel IView.ViewModel
        {
            get { return ViewModel; }
            set { ViewModel = value as TViewModel; }
        }

        #region Clean

        public event EventHandler Clean;

        protected virtual void OnClean(EventArgs e)
        {
            EventHandler L_Handler = Clean;

            if (L_Handler != null)
            {
                L_Handler(this, e);
            }
        }

        #endregion Clean

        #region Disposable

        /// <summary>
        /// Releases all resources used by an instance of the <see cref="ViewControl" /> class.
        /// </summary>
        /// <remarks>
        /// This method calls the virtual <see cref="Dispose(bool)" /> method, passing in
        /// <strong>true</strong>, and then suppresses finalization of the instance.
        /// </remarks>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~ViewControl()
        {
            Dispose(false);
        }

        /// <summary>
        /// Releases the unmanaged resources used by an instance of the <see cref="ViewControl" />
        /// class and optionally releases the managed resources.
        /// </summary>
        /// <param name="P_Disposing"><strong>true</strong> to release both managed and unmanaged
        /// resources; <strong>false</strong> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool P_Disposing)
        {
            if (P_Disposing)
            {
                _LokiAdapter.Dispose();
            }
        }

        #endregion Disposable
    }
}