﻿using System;
using System.Collections.Generic;
using Loki.Engine;

namespace Luna.Data.Configuration
{
    /// <summary>
    /// Configuration data provider.
    /// </summary>
    public interface IConfigurationProvider : IDataProvider
    {
        #region Database management
        /// <summary>
        /// Creates the new store.
        /// </summary>
        /// <param name="P_StorePath">Path of the store.</param>
        void CreateNewStore(string P_StorePath);

        /// <summary>
        /// Opens the existing store.
        /// </summary>
        /// <param name="P_StorePath">Path of the store.</param>
        void OpenExistingStore(string P_StorePath);

        /// <summary>
        /// Moves the current store to the specified location.
        /// </summary>
        /// <param name="P_StorePath">The store new path.</param>
        void MoveCurrentStoreTo(string P_StorePath);
        #endregion

        #region Connectors
        /// <summary>
        /// Gets all connectors.
        /// </summary>
        /// <returns>All defined connectors.</returns>
        IEnumerable<IConnectorData> GetAllConnectors();

        /// <summary>
        /// Gets the connector.
        /// </summary>
        /// <param name="P_ConnectorID">The connector ID.</param>
        /// <returns>The speicifed connector configuration.</returns>
        IConnectorData GetConnector(Guid P_ConnectorID);

        /// <summary>
        /// Saves the connector configuration.
        /// </summary>
        /// <param name="P_Connector">The connector configuration.</param>
        void SaveConnector(IConnectorData P_Connector);

        /// <summary>
        /// Deletes the connector configuration.
        /// </summary>
        /// <param name="P_ConnectorID">The connector ID.</param>
        void DeleteConnector(Guid P_ConnectorID);
        #endregion
    }
}
