﻿using System;
using System.ComponentModel;
using Loki.Data;
using Loki.Utils;
using Luna.Data;

namespace Luna.Model
{
    public class LunaReferenceEntity<TData> : LunaEntity<TData>, ILunaReferenceEntity, IEquatable<IReference>
        where TData : class, IDataEntity<Guid>, IReferenceData
    {
        public LunaReferenceEntity()
            : base()
        {
        }

        public LunaReferenceEntity(LunaReferenceEntity<TData> P_Entity)
            : base(P_Entity)
        {
            BeginInit();
            Google = P_Entity.Google;
            GoogleLastUpdate = P_Entity.GoogleLastUpdate;
            Zoho = P_Entity.Zoho;
            ZohoLastUpdate = P_Entity.ZohoLastUpdate;
            LastUpdate = P_Entity.LastUpdate;
            EndInit(!IsChanged);
        }

        public LunaReferenceEntity(TData P_Data)
            : base(P_Data)
        {
        }

        #region LastUpdate

        private static PropertyChangedEventArgs _LastUpdateChangedArgs = ObservableHelper.CreateChangedArgs<LunaReferenceEntity<TData>>(x => x.LastUpdate);

        private static PropertyChangingEventArgs _LastUpdateChangingArgs = ObservableHelper.CreateChangingArgs<LunaReferenceEntity<TData>>(x => x.LastUpdate);

        public DateTime? LastUpdate
        {
            get
            {
                return Data.LastUpdate;
            }

            set
            {
                if (value != Data.LastUpdate)
                {
                    NotifyChanging(_LastUpdateChangingArgs);
                    Data.LastUpdate = value;
                    NotifyChangedAndDirty(_LastUpdateChangedArgs);
                }
            }
        }

        #endregion LastUpdate

        #region Google

        private static PropertyChangedEventArgs _GoogleChangedArgs = ObservableHelper.CreateChangedArgs<LunaReferenceEntity<TData>>(x => x.Google);

        private static PropertyChangingEventArgs _GoogleChangingArgs = ObservableHelper.CreateChangingArgs<LunaReferenceEntity<TData>>(x => x.Google);

        public string Google
        {
            get
            {
                return Data.Google;
            }

            set
            {
                if (value != Data.Google)
                {
                    NotifyChanging(_GoogleChangingArgs);
                    Data.Google = value;
                    NotifyChangedAndDirty(_GoogleChangedArgs);
                }
            }
        }

        #endregion Google

        #region GoogleLastUpdate

        private static PropertyChangedEventArgs _GoogleLastUpdateChangedArgs = ObservableHelper.CreateChangedArgs<LunaReferenceEntity<TData>>(x => x.GoogleLastUpdate);

        private static PropertyChangingEventArgs _GoogleLastUpdateChangingArgs = ObservableHelper.CreateChangingArgs<LunaReferenceEntity<TData>>(x => x.GoogleLastUpdate);

        public DateTime? GoogleLastUpdate
        {
            get
            {
                return Data.GoogleLastUpdate;
            }

            set
            {
                if (value != Data.GoogleLastUpdate)
                {
                    NotifyChanging(_GoogleLastUpdateChangingArgs);
                    Data.GoogleLastUpdate = value;
                    NotifyChangedAndDirty(_GoogleLastUpdateChangedArgs);
                }
            }
        }

        #endregion GoogleLastUpdate

        #region Zoho

        private static PropertyChangedEventArgs _ZohoChangedArgs = ObservableHelper.CreateChangedArgs<LunaReferenceEntity<TData>>(x => x.Zoho);

        private static PropertyChangingEventArgs _ZohoChangingArgs = ObservableHelper.CreateChangingArgs<LunaReferenceEntity<TData>>(x => x.Zoho);

        public int? Zoho
        {
            get
            {
                return Data.Zohocrm;
            }

            set
            {
                if (value != Data.Zohocrm)
                {
                    NotifyChanging(_ZohoChangingArgs);
                    Data.Zohocrm = value;
                    NotifyChangedAndDirty(_ZohoChangedArgs);
                }
            }
        }

        #endregion Zoho

        #region ZohoLastUpdate

        private static PropertyChangedEventArgs _ZohoLastUpdateChangedArgs = ObservableHelper.CreateChangedArgs<LunaReferenceEntity<TData>>(x => x.ZohoLastUpdate);

        private static PropertyChangingEventArgs _ZohoLastUpdateChangingArgs = ObservableHelper.CreateChangingArgs<LunaReferenceEntity<TData>>(x => x.ZohoLastUpdate);

        public DateTime? ZohoLastUpdate
        {
            get
            {
                return Data.ZohocrmLastUpdate;
            }

            set
            {
                if (value != Data.ZohocrmLastUpdate)
                {
                    NotifyChanging(_ZohoLastUpdateChangingArgs);
                    Data.ZohocrmLastUpdate = value;
                    NotifyChangedAndDirty(_ZohoLastUpdateChangedArgs);
                }
            }
        }

        #endregion ZohoLastUpdate

        public bool Equals(IReference other)
        {
            return other != null && other.ID == ID;
        }

        public override bool Equals(object obj)
        {
            return Equals(obj as IReference);
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }
}