﻿using System;
using System.Collections.Generic;
using Luna.Connectors;
using Luna.Data.Configuration;

namespace Luna.Model.Configuration
{
    public interface IConfigurationService
    {
        /// <summary>
        /// Gets the current database descriptor.
        /// </summary>
        /// <value>The current database descriptor.</value>
        IDatabaseDescriptor Current { get; }

        /// <summary>
        /// Open a new database store.
        /// </summary>
        /// <param name="P_StorePath">Store path.</param>
        void Open(string P_StorePath);

        /// <summary>
        /// Creates a new database store.
        /// </summary>
        void New();

        /// <summary>
        /// Saves the current database in a new store.
        /// </summary>
        /// <param name="P_StorePath">The store path.</param>
        void SaveAs(string P_StorePath);

        /// <summary>
        /// Occurs when current database store has changed.
        /// </summary>
        event EventHandler DatabaseChanged;

        #region Connectors

        IDictionary<ConnectorType, IConnectorConfiguration> ConnectorConfigurations { get; }

        void Configure(IConnectorConfiguration P_Connector);

        void Disable(IConnectorConfiguration P_Connector);

        string Test(IConnectorConfiguration P_Connector);

        #endregion Connectors
    }
}