﻿using Loki.Utils;
using Luna.Data.Contacts;

namespace Luna.Connectors.ZohoCRM
{
    public class ZohoCRMConnector : LokiObject, IConnector
    {
        public ZohoCRMSettingsManager Manager { get; set; }

        public ZohoCRMContactProvider Contacts { get; set; }

        private IZohoCRMSettings _Configuration;

        private bool _Activated = false;

        public ConnectorType Type
        {
            get { return ConnectorType.ZohoCRM; }
        }

        public bool Enabled
        {
            get { return _Configuration != null && _Activated; }
        }

        public string Name
        {
            get { return _Configuration == null ? string.Empty : _Configuration.Login; }
        }

        IContactProvider IConnector.Contacts
        {
            get { return Contacts; }
        }

        public void Configure(IConnectorConfiguration P_Configuration)
        {
            IZohoCRMSettings L_Settings = P_Configuration as IZohoCRMSettings;
            if (L_Settings != null)
            {
                _Configuration = L_Settings;
                if (Test(P_Configuration) == string.Empty)
                {
                    Manager.Configure(L_Settings);
                    _Activated = true;
                }
                else
                {
                    _Activated = false;
                }
            }
            else
            {
                ResetConfig();
            }
        }

        public string Test(IConnectorConfiguration P_Configuration)
        {
            IZohoCRMSettings L_Settings = P_Configuration as IZohoCRMSettings;
            if (L_Settings != null)
            {
                return Contacts.Test(L_Settings);
            }
            else
            {
                return Resources.Errors.INVALID_CONFIGURATION;
            }
        }

        public void ResetConfig()
        {
            _Activated = false;
            _Configuration = null;
        }
    }
}