﻿using System.ComponentModel;
using System.Globalization;
using Loki;
using Loki.Commands;
using Loki.UI;
using Loki.Utils;
using Luna.Model.Contacts;
using Luna.UI.Resources;

namespace Luna.UI.Contacts
{
    public class CategoryEditViewModel : LunaViewModel
    {
        #region CurrentCategory

        private static PropertyChangedEventArgs _CurrentCategoryChangedArgs = ObservableHelper.CreateChangedArgs<CategoryEditViewModel>(x => x.CurrentCategory);

        private static PropertyChangingEventArgs _CurrentCategoryChangingArgs = ObservableHelper.CreateChangingArgs<CategoryEditViewModel>(x => x.CurrentCategory);

        private IEditableCategory _CurrentCategory;

        public IEditableCategory CurrentCategory
        {
            get
            {
                return _CurrentCategory;
            }

            set
            {
                if (value != _CurrentCategory)
                {
                    NotifyChanging(_CurrentCategoryChangingArgs);
                    _CurrentCategory = value;
                    NotifyChangedAndDirty(_CurrentCategoryChangedArgs);
                }
            }
        }

        #endregion CurrentCategory

        #region Name

        private static PropertyChangedEventArgs _NameChangedArgs = ObservableHelper.CreateChangedArgs<CategoryEditViewModel>(x => x.Name);

        private static PropertyChangingEventArgs _NameChangingArgs = ObservableHelper.CreateChangingArgs<CategoryEditViewModel>(x => x.Name);

        private string _Name;

        public string Name
        {
            get
            {
                return _Name;
            }

            set
            {
                if (value != _Name)
                {
                    NotifyChanging(_NameChangingArgs);
                    _Name = value;
                    NotifyChangedAndDirty(_NameChangedArgs);
                }
            }
        }

        #endregion Name

        #region Description

        private static PropertyChangedEventArgs _DescriptionChangedArgs = ObservableHelper.CreateChangedArgs<CategoryEditViewModel>(x => x.Description);

        private static PropertyChangingEventArgs _DescriptionChangingArgs = ObservableHelper.CreateChangingArgs<CategoryEditViewModel>(x => x.Description);

        private string _Description;

        public string Description
        {
            get
            {
                return _Description;
            }

            set
            {
                if (value != _Description)
                {
                    NotifyChanging(_DescriptionChangingArgs);
                    _Description = value;
                    NotifyChangedAndDirty(_DescriptionChangedArgs);
                }
            }
        }

        #endregion Description

        public CategoryEditViewModel()
        {
            Init += CategoryEditViewModel_Init;
            Load += CategoryEditViewModel_Load;
        }

        private void CategoryEditViewModel_Load(object sender, System.EventArgs e)
        {
            RefreshValues();
        }

        private void CategoryEditViewModel_Init(object sender, System.EventArgs e)
        {
            this.WatchChange(this, x => x.CurrentCategory, c => c.CurrentCategory_Changed);

            Commands.Handle(Loki.Commands.ApplicationCommands.SAVE, Save_CanExecute, Save_Execute);

            AddValidator<CategoryEditViewModel, string>(x => x.Name, Rules.Required, "Le nom est requis.");
        }

        private void Save_CanExecute(object P_Sender, CanExecuteCommandEventArgs e)
        {
            e.CanExecute |= IsChanged;
        }

        private void Save_Execute(object P_Sender, CommandEventArgs e)
        {
            if (ValidateData())
            {
                CurrentCategory.Name = Name;
                CurrentCategory.Description = Description;

                ApplicationModel.Contacts.Save(CurrentCategory);

                CurrentCategory = ApplicationModel.Contacts.Categories[CurrentCategory.ID].EditableCopy;

                UserMessage(Resources.Messages.SAVE_GENERIC);
            }
        }

        private void CurrentCategory_Changed(object P_Sender, PropertyChangedEventArgs e)
        {
            RefreshValues();
        }

        private void RefreshValues()
        {
            if (CurrentCategory != null)
            {
                DisplayName = string.Format(CultureInfo.CurrentUICulture, FormNames.CATEGORY_EDIT, CurrentCategory.Name);

                Name = CurrentCategory.Name;
                Description = CurrentCategory.Description;

                ValidateData(ValidationMode.Information);
            }
            else
            {
                Name = null;
                Description = null;
            }
        }
    }
}