﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Reflection;
using System.Text.RegularExpressions;
using Luna.Data.SqlCe.Messages;

namespace Luna.Data.SqlCe
{
    public class LunaDatabaseAdapter : SqlCeDatabaseAdapter
    {
        private const string REQUEST_REGEX = ".([a-zA-Z_]*?).sql";
        private const string DEFAULT_NAMESPACE = "Luna.Data.SqlCe.";

        private Dictionary<string, string> _Requests;

        private static object _RequestLock = new object();

        protected override string GetRequest(string P_CommandName)
        {
            lock (_RequestLock)
            {
                if (_Requests == null)
                {
                    // load requests
                    Assembly L_Assembly = Assembly.GetExecutingAssembly();
                    string[] L_ResourcesNames = L_Assembly.GetManifestResourceNames();
                    _Requests = new Dictionary<string, string>(StringComparer.InvariantCultureIgnoreCase);

                    Regex L_RegEx = new Regex(REQUEST_REGEX);

                    foreach (string L_ResourceName in L_ResourcesNames)
                    {
                        if (L_RegEx.IsMatch(L_ResourceName))
                        {
                            Stream L_File = L_Assembly.GetManifestResourceStream(L_ResourceName);
                            StreamReader L_Reader = new StreamReader(L_File);

                            _Requests[L_RegEx.Match(L_ResourceName).Groups[1].Value] = L_Reader.ReadToEnd();
                            L_Reader.Close();
                        }
                    }
                }

                if (_Requests.ContainsKey(P_CommandName))
                {
                    return _Requests[P_CommandName];
                }
                else
                {
                    throw BuildErrorFormat<ArgumentException>(Errors.LunaDatabaseAdapter_UnavailableRequest, P_CommandName, this.DataSourceDescription);
                }
            }
        }

        public override bool Initialized
        {
            get { return File.Exists(FileName); }
        }

        public override void Initialize(NameValueCollection P_ConfigParams)
        {
        }
    }
}
