﻿using System;
using Loki.Data;
using Loki.Engine;

namespace Luna.Data.Test
{
    public abstract class DataWithReferenceTest<TService, TEntity> : BaseTest<TService, TEntity, IReferenceData>
        where TService : class, IDataProvider
        where TEntity : class, IDataEntity<Guid>, IReferenceData
    {
        public override Func<TEntity, IReferenceData> KeyGetter
        {
            get { return x => x; }
        }

        public override bool AreSame(TEntity P_Old, TEntity P_Entity2)
        {
            bool L_Return = true;

            L_Return &= P_Old.Google == P_Entity2.Google;
            L_Return &= P_Old.GoogleLastUpdate == P_Entity2.GoogleLastUpdate;
            L_Return &= (P_Old.LastUpdate - P_Entity2.LastUpdate) < TimeSpan.FromMilliseconds(2);
            L_Return &= P_Old.PK == P_Entity2.PK;
            L_Return &= P_Old.Zohocrm == P_Entity2.Zohocrm;
            L_Return &= P_Old.ZohocrmLastUpdate == P_Entity2.ZohocrmLastUpdate;

            return L_Return;
        }

        protected const string GOOGLE_ID = "Google";

        public virtual void LoadEntity(IReferenceData P_Data)
        {
            P_Data.PK = Guid.NewGuid();
            P_Data.AlreadyPersisted = false;
            P_Data.Google = GetString(GOOGLE_ID, 32);
            P_Data.GoogleLastUpdate = GetDate();
            P_Data.LastUpdate = GetDate();
            P_Data.Zohocrm = GetInteger();
            P_Data.ZohocrmLastUpdate = GetDate();
        }
    }
}