﻿using System.Data;
using Loki.Data;
using Luna.Connectors;

namespace Luna.Data
{
    public class ReferenceDataMapper<TDataEntity> : LunaMapper<TDataEntity, IReferenceData> where TDataEntity : IReferenceData
    {
        private const string LAST_UPDATE = "LastUpdate";
        private const string GOOGLE = "Google";
        private const string GOOGLE_LAST_UPDATE = "GoogleLastUpdate";
        private const string ZOHOCRM = "Zohocrm";
        private const string ZOHOCRM_LAST_UPDATE = "ZohocrmLastUpdate";
        private const string FROM = "From";
        private const string VALIDATED = "Validated";

        protected override void LoadCoreData(IReferenceData P_Entity, IDataRecord P_Record)
        {
            P_Entity.LastUpdate = P_Record.GetNullableDateAndTime(LAST_UPDATE);
            P_Entity.Google = P_Record.GetString(GOOGLE);
            P_Entity.GoogleLastUpdate = P_Record.GetNullableDateAndTime(GOOGLE_LAST_UPDATE);
            P_Entity.Zohocrm = P_Record.GetNullableInteger(ZOHOCRM);
            P_Entity.ZohocrmLastUpdate = P_Record.GetNullableDateAndTime(ZOHOCRM_LAST_UPDATE);
            P_Entity.From = P_Record.GetNullableEnum<ConnectorType>(FROM);
            P_Entity.Validated = P_Record.GetBoolean(VALIDATED);
        }

        protected override void MapSaveParameters(IDatabaseCommand P_Command, IReferenceData P_Entity)
        {
            P_Command.AddParameter(LAST_UPDATE, P_Entity.LastUpdate);
            P_Command.AddParameter(GOOGLE, P_Entity.Google);
            P_Command.AddParameter(GOOGLE_LAST_UPDATE, P_Entity.GoogleLastUpdate);
            P_Command.AddParameter(ZOHOCRM, P_Entity.Zohocrm);
            P_Command.AddParameter(ZOHOCRM_LAST_UPDATE, P_Entity.ZohocrmLastUpdate);
            P_Command.AddParameter(FROM, P_Entity.From);
            P_Command.AddParameter(VALIDATED, P_Entity.Validated);
            P_Command.AddParameter(IDParameterName, P_Entity.PK);
        }
    }
}