﻿using System;
using System.Collections.Generic;

namespace Luna.Data.Configuration
{
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1063:ImplementIDisposableCorrectly", Justification = "False positive")]
    public class ConfigurationProvider : LunaDataProvider, IConfigurationProvider
    {
        public ConnectorDataMapper ConnectorMapper { get; set; }

        protected IDatabaseManager Manager
        {
            get
            {
                return DataContext.Get<IDatabaseManager>();
            }
        }

        #region Store Management

        public void CreateNewStore(string P_StoreName)
        {
            Manager.Create(P_StoreName);
        }

        public void OpenExistingStore(string P_StoreName)
        {
            Manager.Open(P_StoreName);
        }

        public void MoveCurrentStoreTo(string P_StoreName)
        {
            Manager.SaveAs(P_StoreName);
        }

        #endregion Store Management

        #region Connectors

        public IEnumerable<IConnectorData> GetAllConnectors()
        {
            return WrapperTransactional(T => ConnectorMapper.GetAll(T));
        }

        public IConnectorData GetConnector(Guid P_ConnectorID)
        {
            return WrapperTransactional(T => ConnectorMapper.Get(T, P_ConnectorID));
        }

        public void SaveConnector(IConnectorData P_Connector)
        {
            WrapperTransactional(T => ConnectorMapper.Save(T, P_Connector));
        }

        public void DeleteConnector(Guid P_ConnectorID)
        {
            WrapperTransactional(T => ConnectorMapper.Delete(T, P_ConnectorID));
        }

        #endregion Connectors
    }
}