﻿using System;
using System.Linq;
using Loki.Engine;
using Loki.Model;
using Luna.Data;

namespace Luna.Model.Contacts
{
    internal class CacheConfiguration : CacheConfiguration<IDataContext>
    {
        public static readonly ContainerConfiguration<Account, Guid> ACCOUNTS = new ContainerConfiguration<Account, Guid>()
        {
            FullLoader = () => DataContext.Contacts.AccountGetAll().Select(x => new Account(x)),
            PrimaryKey = new UniqueKeyConfiguration<Account, Guid>()
            {
                Indexer = x => x.ID,
                Loader = x => { var L_Return = DataContext.Contacts.AccountGet(CreateReference(x)); return L_Return != null ? new Account(L_Return) : null; }
            }
        };

        public static readonly ContainerConfiguration<Category, Guid> CATEGORIES = new ContainerConfiguration<Category, Guid>()
        {
            FullLoader = () => DataContext.Contacts.CategoryGetAll().Select(x => new Category(x)),
            PrimaryKey = new UniqueKeyConfiguration<Category, Guid>()
            {
                Indexer = x => x.ID,
                Loader = x => { var L_Return = DataContext.Contacts.CategoryGet(CreateReference(x)); return L_Return != null ? new Category(L_Return) : null; }
            }
        };

        private static IReferenceData CreateReference(Guid P_ID)
        {
            var L_Return = DataContext.Create<IReferenceData>();
            L_Return.PK = P_ID;
            return L_Return;
        }
    }
}