﻿using System.ComponentModel;
using System.Text;
using Loki.Utils;
using Luna.Data.Contacts;

namespace Luna.Model.Contacts
{
    internal class Account : LunaReferenceEntity<IAccountData>, IAccount
    {
        #region Name

        private static PropertyChangedEventArgs _NameChangedArgs = ObservableHelper.CreateChangedArgs<Account>(x => x.Name);

        private static PropertyChangingEventArgs _NameChangingArgs = ObservableHelper.CreateChangingArgs<Account>(x => x.Name);

        public string Name
        {
            get
            {
                return Data.Name;
            }

            set
            {
                if (value != Data.Name)
                {
                    NotifyChanging(_NameChangingArgs);
                    Data.Name = value;
                    NotifyChangedAndDirty(_NameChangedArgs);
                }
            }
        }

        #endregion Name

        #region Address

        private static PropertyChangedEventArgs _AddressChangedArgs = ObservableHelper.CreateChangedArgs<Account>(x => x.Address);

        private static PropertyChangingEventArgs _AddressChangingArgs = ObservableHelper.CreateChangingArgs<Account>(x => x.Address);

        public string Address
        {
            get
            {
                return Data.Address;
            }

            set
            {
                if (value != Data.Address)
                {
                    NotifyChanging(_AddressChangingArgs);
                    Data.Address = value;
                    NotifyChangedAndDirty(_AddressChangedArgs);
                }
            }
        }

        #endregion Address

        #region Zipcode

        private static PropertyChangedEventArgs _ZipcodeChangedArgs = ObservableHelper.CreateChangedArgs<Account>(x => x.Zipcode);

        private static PropertyChangingEventArgs _ZipcodeChangingArgs = ObservableHelper.CreateChangingArgs<Account>(x => x.Zipcode);

        public string Zipcode
        {
            get
            {
                return Data.Zipcode;
            }

            set
            {
                if (value != Data.Zipcode)
                {
                    NotifyChanging(_ZipcodeChangingArgs);
                    Data.Zipcode = value;
                    NotifyChangedAndDirty(_ZipcodeChangedArgs);
                }
            }
        }

        #endregion Zipcode

        #region City

        private static PropertyChangedEventArgs _CityChangedArgs = ObservableHelper.CreateChangedArgs<Account>(x => x.City);

        private static PropertyChangingEventArgs _CityChangingArgs = ObservableHelper.CreateChangingArgs<Account>(x => x.City);

        public string City
        {
            get
            {
                return Data.City;
            }

            set
            {
                if (value != Data.City)
                {
                    NotifyChanging(_CityChangingArgs);
                    Data.City = value;
                    NotifyChangedAndDirty(_CityChangedArgs);
                }
            }
        }

        #endregion City

        #region Country

        private static PropertyChangedEventArgs _CountryChangedArgs = ObservableHelper.CreateChangedArgs<Account>(x => x.Country);

        private static PropertyChangingEventArgs _CountryChangingArgs = ObservableHelper.CreateChangingArgs<Account>(x => x.Country);

        public string Country
        {
            get
            {
                return Data.Country;
            }

            set
            {
                if (value != Data.Country)
                {
                    NotifyChanging(_CountryChangingArgs);
                    Data.Country = value;
                    NotifyChangedAndDirty(_CountryChangedArgs);
                }
            }
        }

        #endregion Country

        #region PhoneNumber

        private static PropertyChangedEventArgs _PhoneNumberChangedArgs = ObservableHelper.CreateChangedArgs<Account>(x => x.PhoneNumber);

        private static PropertyChangingEventArgs _PhoneNumberChangingArgs = ObservableHelper.CreateChangingArgs<Account>(x => x.PhoneNumber);

        public string PhoneNumber
        {
            get
            {
                return FormatPhoneNumber(Data.PhoneNumber);
            }

            set
            {
                if (value != Data.PhoneNumber)
                {
                    NotifyChanging(_PhoneNumberChangingArgs);
                    Data.PhoneNumber = CleanPhoneNumber(value);
                    NotifyChangedAndDirty(_PhoneNumberChangedArgs);
                }
            }
        }

        private string CleanPhoneNumber(string P_Value)
        {
            if (string.IsNullOrWhiteSpace(P_Value))
            {
                return P_Value;
            }

            StringBuilder L_Builder = new StringBuilder();
            bool L_NumberFinded = false;
            foreach (var L_Char in P_Value)
            {
                if (L_Char == '+' && !L_NumberFinded || char.IsDigit(L_Char))
                {
                    L_Builder.Append(L_Char);
                }

                L_NumberFinded |= char.IsDigit(L_Char);
            }

            return L_Builder.ToString();
        }

        private string FormatPhoneNumber(string P_Value)
        {
            if (string.IsNullOrWhiteSpace(P_Value))
            {
                return P_Value;
            }

            StringBuilder L_Builder = new StringBuilder();
            int L_SpaceIncluded = 0;
            for (int i = 0; i < P_Value.Length; i++)
            {
                L_Builder.Insert(0, P_Value[P_Value.Length - 1 - i]);
                if (i % 2 == 1 && L_SpaceIncluded < 4)
                {
                    L_Builder.Insert(0, " ");
                    L_SpaceIncluded++;
                }
            }

            return L_Builder.ToString();
        }

        #endregion PhoneNumber

        #region Url

        private static PropertyChangedEventArgs _UrlChangedArgs = ObservableHelper.CreateChangedArgs<Account>(x => x.Url);

        private static PropertyChangingEventArgs _UrlChangingArgs = ObservableHelper.CreateChangingArgs<Account>(x => x.Url);

        public string Url
        {
            get
            {
                return Data.Url;
            }

            set
            {
                if (value != Data.Url)
                {
                    NotifyChanging(_UrlChangingArgs);
                    Data.Url = value;
                    NotifyChangedAndDirty(_UrlChangedArgs);
                }
            }
        }

        #endregion Url

        #region Mail

        private static PropertyChangedEventArgs _MailChangedArgs = ObservableHelper.CreateChangedArgs<Account>(x => x.Mail);

        private static PropertyChangingEventArgs _MailChangingArgs = ObservableHelper.CreateChangingArgs<Account>(x => x.Mail);

        public string Mail
        {
            get
            {
                return Data.Mail;
            }

            set
            {
                if (value != Data.Mail)
                {
                    NotifyChanging(_MailChangingArgs);
                    Data.Mail = value;
                    NotifyChangedAndDirty(_MailChangedArgs);
                }
            }
        }

        #endregion Mail

        public Account()
            : base()
        {
            Name = "Nouveau compte";
        }

        public Account(Account P_Base)
            : base(P_Base)
        {
        }

        public Account(IAccountData P_Data)
            : base(P_Data)
        {
        }
    }
}