﻿using System;
using System.Collections.Generic;
using System.Linq;
using Loki.Data;
using Loki.Model;
using Luna.Connectors;
using Luna.Data.Contacts;

namespace Luna.Model.Contacts
{
    public class LunaContactService : LunaService, IContactService
    {
        public void Reset()
        {
            _Categories.Reset();
        }

        public LunaContactService()
        {
            _Accounts = CreateContainer<Account, Guid>(CacheConfiguration.ACCOUNTS);
            _Categories = CreateContainer<Category, Guid>(CacheConfiguration.CATEGORIES);
        }

        public IEnumerable<IContact> GetAll()
        {
            return DataContext.Connectors[ConnectorType.Google].Contacts.ContactGetAll().Select(x => new Contact(x));
        }

        private EntityContainer<Account, Guid> _Accounts;

        public IIndexedEnumerable<IAccount, Guid> Accounts
        {
            get { return _Accounts; }
        }

        public void Save(IAccount P_Account)
        {
            Account L_Account = P_Account as Account;
            if (L_Account != null)
            {
                DataContext.Contacts.AccountSave(L_Account.Data);
            }

            _Accounts.Reset();
        }

        public void Delete(IAccount P_Account)
        {
            Account L_Account = P_Account as Account;
            if (L_Account != null)
            {
                DataContext.Contacts.AccountDelete(L_Account.Data);
            }
        }

        private EntityContainer<Category, Guid> _Categories;

        public IIndexedEnumerable<ICategory, Guid> Categories
        {
            get { return _Categories; }
        }

        public void Save(IEditableCategory P_Category)
        {
            Save<IEditableCategory, Category, ICategoryData>(P_Category, DataContext.Contacts.CategorySave, _Categories);
        }

        private void Save<TInterface, TEntity, TEntityData>(TInterface P_Entity, Action<TEntityData> P_DataSave, EntityContainer<TEntity, Guid> P_Store)
            where TEntity : LunaEntity<TEntityData>, TInterface, IAtomicEntity<Guid>
            where TEntityData : class, IDataEntity<Guid>
        {
            TEntity L_Entity = P_Entity as TEntity;
            if (L_Entity != null && L_Entity.IsChanged)
            {
                P_DataSave(L_Entity.Data);

                P_Store.Register(L_Entity);
            }
        }

        private void Delete<TInterface, TEntity, TEntityData>(TInterface P_Entity, Action<TEntityData> P_DataDelete, EntityContainer<TEntity, Guid> P_Store)
            where TEntity : LunaEntity<TEntityData>, TInterface, IAtomicEntity<Guid>
            where TEntityData : class, IDataEntity<Guid>
        {
            TEntity L_Entity = P_Entity as TEntity;
            if (L_Entity != null)
            {
                P_DataDelete(L_Entity.Data);

                P_Store.Remove(L_Entity);
            }
        }

        public void Delete(IEditableCategory P_Category)
        {
            Delete<IEditableCategory, Category, ICategoryData>(P_Category, DataContext.Contacts.CategoryDelete, _Categories);
        }
    }
}