﻿using System;
using System.Collections.Generic;
using System.Linq;
using Luna.Data;
using Luna.Data.Contacts;
using Luna.Model.Contacts;
using Rhino.Mocks;

namespace Luna.Services.Test.Contacts
{
    public class CategoryTest : ReferenceEntityTest<IContactProvider, IEditableCategory, ICategoryData>
    {
        public override void SetUp()
        {
            base.SetUp();
            ApplicationModel.Contacts.Reset();
            PropertyShifters.Add(new PropertyShifter<IEditableCategory, string>(x => x.Name, DataManager.GetString));
            PropertyShifters.Add(new PropertyShifter<IEditableCategory, string>(x => x.Description, DataManager.GetString));
        }

        protected override ICategoryData CreateMockData()
        {
            return ContactDataManager.CreateCategoryData();
        }

        protected override Func<IContactProvider, IEnumerable<ICategoryData>> DataGetAll
        {
            get { return p => p.CategoryGetAll(); }
        }

        protected override Func<IContactProvider, IReferenceData, ICategoryData> DataGet
        {
            get { return (p, k) => p.CategoryGet(k); }
        }

        protected override Action<IEditableCategory> EntityDelete
        {
            get { return ApplicationModel.Contacts.Delete; }
        }

        protected override Action<IContactProvider, IReferenceData> DataDelete
        {
            get { return (p, k) => p.CategoryDelete(k); }
        }

        protected override Action<IEditableCategory> EntitySave
        {
            get { return ApplicationModel.Contacts.Save; }
        }

        protected override Action<IContactProvider, ICategoryData> DataSave
        {
            get { return (p, i) => p.CategorySave(i); }
        }

        protected override IContactProvider DataProvider
        {
            get { return DataContext.Contacts; }
        }

        protected override IEnumerable<IEditableCategory> EntitiesAll
        {
            get { return ApplicationModel.Contacts.Categories.Select(x => x.EditableCopy); }
        }

        protected override IEditableCategory EntityGet(Guid P_Key)
        {
            return ApplicationModel.Contacts.Categories[P_Key].EditableCopy;
        }

        protected override ICategoryData SaveMatch(IEditableCategory P_Entity)
        {
            return Arg<ICategoryData>.Matches(
                 a => DataKeyMatch(a, P_Entity)
                     && a.Name == P_Entity.Name
                     && a.Description == P_Entity.Description);
        }

        protected override bool BuildMatch(IEditableCategory P_Entity, ICategoryData P_Data)
        {
            return base.BuildMatch(P_Entity, P_Data) && P_Entity.Name == P_Data.Name && P_Entity.Description == P_Data.Description;
        }
    }
}