﻿using System;
using System.ComponentModel;
using Loki;
using Luna.Data.Project;

namespace Luna.Model.Project
{
    internal class ProjectTask : LunaEntity, IProjectTask, IProjectTaskData
    {
        #region  ProjectID
        private static PropertyChangedEventArgs _ProjectIDChangeArgs = ObservableHelper.CreateArgs<ProjectTask>(x => x.ProjectID);

        private Guid _ProjectID;

        public Guid ProjectID
        {
            get
            {
                return _ProjectID;
            }

            set
            {
                if (value != _ProjectID)
                {
                    _ProjectID = value;
                    NotifyChangedAndDirty(_ProjectIDChangeArgs);
                }
            }
        }
        #endregion

        private IProject _Project;

        public IProject Project
        {
            get
            {
                if (_Project == null)
                {
                    return ApplicationModel.Project.Projects[_ProjectID];
                }
                else
                {
                    return _Project;
                }
            }

            set
            {
                if (value == null)
                {
                    ProjectID = Guid.Empty;
                }
                else
                {
                    ProjectID = value.ID;
                }

                _Project = value;
            }
        }

        #region  Code
        private static PropertyChangedEventArgs _CodeChangeArgs = ObservableHelper.CreateArgs<ProjectTask>(x => x.Code);

        private string _Code;

        public string Code
        {
            get
            {
                return _Code;
            }

            set
            {
                if (value != _Code)
                {
                    _Code = value;
                    NotifyChangedAndDirty(_CodeChangeArgs);
                }
            }
        }
        #endregion

        #region  Name
        private static PropertyChangedEventArgs _NameChangeArgs = ObservableHelper.CreateArgs<ProjectTask>(x => x.Name);

        private string _Name;

        public string Name
        {
            get
            {
                return _Name;
            }

            set
            {
                if (value != _Name)
                {
                    _Name = value;
                    NotifyChangedAndDirty(_NameChangeArgs);
                }
            }
        }
        #endregion

        #region  Start
        private static PropertyChangedEventArgs _StartChangeArgs = ObservableHelper.CreateArgs<ProjectTask>(x => x.Start);

        private DateTime _Start;

        public DateTime Start
        {
            get
            {
                return _Start;
            }

            set
            {
                if (value != _Start)
                {
                    _Start = value;
                    NotifyChangedAndDirty(_StartChangeArgs);
                }
            }
        }
        #endregion

        #region  End
        private static PropertyChangedEventArgs _EndChangeArgs = ObservableHelper.CreateArgs<ProjectTask>(x => x.End);

        private DateTime _End;

        public DateTime End
        {
            get
            {
                return _End;
            }

            set
            {
                if (value != _End)
                {
                    _End = value;
                    NotifyChangedAndDirty(_EndChangeArgs);
                }
            }
        }
        #endregion

        #region  Budget
        private static PropertyChangedEventArgs _BudgetChangeArgs = ObservableHelper.CreateArgs<ProjectTask>(x => x.Budget);

        private decimal _Budget;

        public decimal Budget
        {
            get
            {
                return _Budget;
            }

            set
            {
                if (value != _Budget)
                {
                    _Budget = value;
                    NotifyChangedAndDirty(_BudgetChangeArgs);
                }
            }
        }
        #endregion

        public Guid PK
        {
            get { return ID; }
            set { ID = value; }
        }

        public ProjectTask()
            : base()
        {
        }

        public ProjectTask(IProjectTask P_Task)
            : base(P_Task)
        {
            Code = P_Task.Code;
            Name = P_Task.Name;
            Start = P_Task.Start;
            End = P_Task.End;
            Budget = P_Task.Budget;
            Project = P_Task.Project;
        }

        public ProjectTask(IProjectTaskData P_Data)
            : base(P_Data)
        {
            _Code = P_Data.Code;
            _Name = P_Data.Name;
            _Start = P_Data.Start;
            _End = P_Data.End;
            _Budget = P_Data.Budget;
            _ProjectID = P_Data.ProjectID;
        }
    }
}
