﻿using System;
using System.Collections.Generic;
using Loki;
using Loki.Engine;
using Loki.Model;
using Luna.Data;
using Luna.Data.Project;

namespace Luna.Model.Project
{
    public class ProjectService : ApplicationService<LunaModel, LunaDataModel>, IProjectService
    {
        public ProjectService()
        {
            DefineTypeCreation<IEditableProject, Project>();
            DefineTypeCreation<IProjectTask, ProjectTask>();

            _Projects = CreateContainer<IProject, Guid>(CacheConfiguration.PROJECTS);
            _Tasks = CreateContainer<ProjectTask, Guid>(CacheConfiguration.TASKS);
        }

        protected IProjectDataProvider DataProvider
        {
            get { return DataContext.Project; }
        }

        private const string _ID = "{7C96024F-38E2-4B00-AE73-0BB65F8B3936}";

        private const string NAME = "Luna project service";

        public override Guid ID
        {
            get { return new Guid(_ID); }
        }

        public override string Name
        {
            get { return NAME; }
        }

        #region Tasks
        private EntityContainer<ProjectTask, Guid> _Tasks;

        private static readonly string INDEX_TASK_PROJECTID = ExpressionHelper.Reflect<ProjectTask>.GetProperty(x => x.ProjectID).Name;

        public IEnumerable<IProjectTask> GetTasks(Guid P_ProjectID)
        {
            return _Tasks[INDEX_TASK_PROJECTID, P_ProjectID];
        }
        #endregion

        #region Projects

        private EntityContainer<IProject, Guid> _Projects;

        public IIndexedEnumerable<IProject, Guid> Projects
        {
            get { return _Projects; }
        }

        public void Save(IEditableProject P_Project)
        {
            Project L_Project = P_Project as Project;
            if (L_Project != null)
            {
                List<IProjectTaskData> L_Tasks = new List<IProjectTaskData>();
                foreach (var L_Task in L_Project.Tasks)
                {
                    IProjectTaskData L_DTask = DataProvider.Create<IProjectTaskData>();
                    L_DTask.Budget = L_Task.Budget;
                    L_DTask.Code = L_Task.Code;
                    L_DTask.End = L_Task.End;
                    L_DTask.Name = L_Task.Name;
                    L_DTask.PK = L_Task.ID;
                    L_DTask.ProjectID = L_Project.ID;
                    L_DTask.Start = L_Task.Start;

                    L_Tasks.Add(L_DTask);
                }

                DataProvider.SaveProject(L_Project, L_Tasks);

                _Tasks.ReloadKey(x => x.ProjectID, L_Project.ID);
                _Projects.Register(new Project(DataProvider.GetProject(L_Project.ID)));
            }
        }

        public bool CanDelete(IEditableProject P_Project)
        {
            return true;
        }

        public void Delete(IEditableProject P_Project)
        {
            DataProvider.DeleteProject(P_Project.ID);
            _Projects.Remove(P_Project);
        }
        #endregion
    }
}
