﻿using System;

namespace Loki.UI
{
    /// <summary>
    /// Base interface for views.
    /// </summary>
    public interface IView : IDisposable
    {
        /// <summary>
        /// Gets or sets the view model.
        /// </summary>
        /// <value>
        /// The view model.
        /// </value>
        IViewModel ViewModel { get; set; }

        /// <summary>
        /// Occurs when the view is disposed.
        /// </summary>
        event EventHandler Disposed;

        /// <summary>
        /// Shows this instance.
        /// </summary>
        void Show();
    }

    /// <summary>
    /// Base interface for views.
    /// </summary>
    /// <typeparam name="TViewModel">The type of the view model.</typeparam>
    public interface IView<TViewModel> : IDisposable where TViewModel : IViewModel
    {
        /// <summary>
        /// Gets or sets the view model.
        /// </summary>
        /// <value>
        /// The view model.
        /// </value>
        TViewModel ViewModel { get; set; }
    }
}
