﻿using System;
using System.Runtime.Serialization;

namespace Loki.Utils
{
    /// <summary>
    /// Loki base exception class.
    /// </summary>
    [Serializable]
    public class LokiException : Exception
    {
        /// <summary>
        /// Gets or sets the error code.
        /// </summary>
        /// <value>The error code.</value>
        public int Code
        {
            get;
            set;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LokiException"/> class.
        /// </summary>
        public LokiException()
            : base()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LokiException"/> class.
        /// </summary>
        /// <param name="P_Message">The error message.</param>
        public LokiException(string P_Message)
            : base(P_Message)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LokiException"/> class.
        /// </summary>
        /// <param name="P_Message">The error message.</param>
        /// <param name="P_InnerException">Inner exception.</param>
        public LokiException(string P_Message, Exception P_InnerException)
            : base(P_Message, P_InnerException)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LokiException"/> class.
        /// </summary>
        /// <param name="P_Info">The object that holds the serialized object data.</param>
        /// <param name="P_Context">The contextual information about the source or destination.</param>
        protected LokiException(SerializationInfo P_Info, StreamingContext P_Context)
            : base(P_Info, P_Context)
        {
        }

        public override void GetObjectData(SerializationInfo P_Info, StreamingContext P_Context)
        {
            base.GetObjectData(P_Info, P_Context);
        }

        /// <summary>
        /// Creates and returns a string representation of the current exception.
        /// </summary>
        /// <returns>
        /// A string representation of the current exception.
        /// </returns>
        /// <PermissionSet>
        /// <IPermission class="System.Security.Permissions.FileIOPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" PathDiscovery="*AllFiles*"/>
        /// </PermissionSet>
        public override string ToString()
        {
            System.Text.StringBuilder L_SBuilder = new System.Text.StringBuilder(128);

            // Make sure we get the standard output
            L_SBuilder.Append(base.ToString());

            // Since the standard output can be nothing, we only add our variables when necessary
            if (L_SBuilder.Length > 0)
            {
                L_SBuilder.Append(Environment.NewLine);
                L_SBuilder.AppendFormat(FORMATTER, Code, Message);
            }

            return L_SBuilder.ToString();
        }

        private const string FORMATTER = "Code {0} - Message {1}";
    }
}
