﻿using System;
using System.ComponentModel;
using Loki;

namespace Luna.UI.Accounting
{
    public class AccountingGraphViewModel : LunaContainerViewModel
    {
        public AccountingGraphViewModel()
        {
            Init += AccountingGraphViewModel_Init;
            Load += AccountingGraphViewModel_Load;

            SourceAccounts = CreateCollection<DisplayNamedEntity>();
        }

        private void AccountingGraphViewModel_Load(object P_Sender, EventArgs e)
        {
            StartDate = new DateTime(DateTime.Today.Year, 01, 01);
            EndDate = DateTime.Today;
            Synchronizer.FromAccounts(SourceAccounts);

            GraphChanged(this, EventArgs.Empty);
        }

        private void AccountingGraphViewModel_Init(object P_Sender, EventArgs e)
        {
            this.WatchChange(this, v => v.GraphType, v => v.GraphChanged);
            this.WatchChange(this, v => v.StartDate, v => v.PropagateValues);
            this.WatchChange(this, v => v.EndDate, v => v.PropagateValues);
            this.WatchChange(this, v => v.SelectedAccountID, v => v.PropagateValues);
        }

        #region  SelectedAccountID
        private static PropertyChangedEventArgs _SelectedAccountIDChangeArgs = ObservableHelper.CreateArgs<AccountingGraphViewModel>(x => x.SelectedAccountID);

        private Guid? _SelectedAccountID;

        public Guid? SelectedAccountID
        {
            get
            {
                return _SelectedAccountID;
            }

            set
            {
                if (value != _SelectedAccountID)
                {
                    _SelectedAccountID = value;
                    NotifyChanged(_SelectedAccountIDChangeArgs);
                }
            }
        }
        #endregion

        #region  StartDate
        private static PropertyChangedEventArgs _StartDateChangeArgs = ObservableHelper.CreateArgs<AccountingGraphViewModel>(x => x.StartDate);

        private DateTime _StartDate;

        public DateTime StartDate
        {
            get
            {
                return _StartDate;
            }

            set
            {
                if (value != _StartDate)
                {
                    _StartDate = value;
                    NotifyChanged(_StartDateChangeArgs);
                }
            }
        }
        #endregion

        #region  EndDate
        private static PropertyChangedEventArgs _EndDateChangeArgs = ObservableHelper.CreateArgs<AccountingGraphViewModel>(x => x.EndDate);

        private DateTime _EndDate;

        public DateTime EndDate
        {
            get
            {
                return _EndDate;
            }

            set
            {
                if (value != _EndDate)
                {
                    _EndDate = value;
                    NotifyChanged(_EndDateChangeArgs);
                }
            }
        }
        #endregion

        #region  GraphType
        private static PropertyChangedEventArgs _GraphTypeChangeArgs = ObservableHelper.CreateArgs<AccountingGraphViewModel>(x => x.GraphType);

        private AccountingGraphType _GraphType;

        public AccountingGraphType GraphType
        {
            get
            {
                return _GraphType;
            }

            set
            {
                if (value != _GraphType)
                {
                    _GraphType = value;
                    NotifyChanged(_GraphTypeChangeArgs);
                }
            }
        }
        #endregion

        public LokiCollection<DisplayNamedEntity> SourceAccounts { get; private set; }

        #region  CurrentGraph
        private static PropertyChangedEventArgs _CurrentGraphChangeArgs = ObservableHelper.CreateArgs<AccountingGraphViewModel>(x => x.CurrentGraph);

        private AccountingGraph _CurrentGraph;

        public AccountingGraph CurrentGraph
        {
            get
            {
                return _CurrentGraph;
            }

            set
            {
                if (value != _CurrentGraph)
                {
                    _CurrentGraph = value;
                    NotifyChanged(_CurrentGraphChangeArgs);
                }
            }
        }
        #endregion

        private void GraphChanged(object P_Sender, EventArgs e)
        {
            if (CurrentGraph != null)
            {
                CurrentGraph.Dispose();
                CurrentGraph.Close.Execute(null);
            }

            CurrentGraph = null;

            switch (GraphType)
            {
                case AccountingGraphType.Level:
                    CurrentGraph = AddNew<AccountingLevelGraphViewModel>();
                    PropagateValues(P_Sender, e);
                    break;
                case AccountingGraphType.Category:
                    CurrentGraph = AddNew<AccountingCategoryGraphViewModel>();
                    PropagateValues(P_Sender, e);
                    break;
                case AccountingGraphType.Projects:
                    CurrentGraph = AddNew<AccountingProjectGraphViewModel>();
                    PropagateValues(P_Sender, e);
                    break;
                default:
                    break;
            }
        }

        private void PropagateValues(object P_Sender, EventArgs e)
        {
            if (CurrentGraph != null)
            {
                CurrentGraph.StartDate = StartDate;
                CurrentGraph.EndDate = EndDate;
                CurrentGraph.AccountID = SelectedAccountID;
            }
        }
    }
}
