﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using Loki;
using Luna.Data.Project;

namespace Luna.Model.Project
{
    internal class Project : LunaEntity, IProject, IProjectData, IEditableProject
    {
        public Guid PK
        {
            get { return ID; }
            set { ID = value; }
        }

        #region  Name
        private static PropertyChangedEventArgs _NameChangeArgs = ObservableHelper.CreateArgs<Project>(x => x.Name);

        private string _Name;

        public string Name
        {
            get
            {
                return _Name;
            }

            set
            {
                if (value != _Name)
                {
                    _Name = value;
                    NotifyChangedAndDirty(_NameChangeArgs);
                }
            }
        }
        #endregion

        #region  Description
        private static PropertyChangedEventArgs _DescriptionChangeArgs = ObservableHelper.CreateArgs<Project>(x => x.Description);

        private string _Description;

        public string Description
        {
            get
            {
                return _Description;
            }

            set
            {
                if (value != _Description)
                {
                    _Description = value;
                    NotifyChangedAndDirty(_DescriptionChangeArgs);
                }
            }
        }
        #endregion

        #region  Code
        private static PropertyChangedEventArgs _CodeChangeArgs = ObservableHelper.CreateArgs<Project>(x => x.Code);

        private string _Code;

        public string Code
        {
            get
            {
                return _Code;
            }

            set
            {
                if (value != _Code)
                {
                    _Code = value;
                    NotifyChangedAndDirty(_CodeChangeArgs);
                }
            }
        }
        #endregion

        #region  Start
        private static PropertyChangedEventArgs _StartChangeArgs = ObservableHelper.CreateArgs<Project>(x => x.Start);

        private DateTime _Start;

        public DateTime Start
        {
            get
            {
                return _Start;
            }

            set
            {
                if (value != _Start)
                {
                    _Start = value;
                    NotifyChangedAndDirty(_StartChangeArgs);
                }
            }
        }
        #endregion

        #region  End
        private static PropertyChangedEventArgs _EndChangeArgs = ObservableHelper.CreateArgs<Project>(x => x.End);

        private DateTime _End;

        public DateTime End
        {
            get
            {
                return _End;
            }

            set
            {
                if (value != _End)
                {
                    _End = value;
                    NotifyChangedAndDirty(_EndChangeArgs);
                }
            }
        }
        #endregion

        #region  Budget
        private static PropertyChangedEventArgs _BudgetChangeArgs = ObservableHelper.CreateArgs<Project>(x => x.Budget);

        private decimal _Budget;

        public decimal Budget
        {
            get
            {
                return _Budget;
            }

            set
            {
                if (value != _Budget)
                {
                    _Budget = value;
                    NotifyChangedAndDirty(_BudgetChangeArgs);
                }
            }
        }
        #endregion

        private LokiCollection<IProjectTask> _Tasks;

        public IEnumerable<IProjectTask> Tasks
        {
            get
            {
                if (_Tasks == null)
                {
                    return ApplicationModel.Project.GetTasks(this.ID);
                }
                else
                {
                    return _Tasks;
                }
            }
        }

        public void Add(IProjectTask P_Task)
        {
            P_Task.Project = this;
            _Tasks.Add(P_Task);
        }

        public void Remove(IProjectTask P_Task)
        {
            P_Task.Project = null;
            _Tasks.Remove(P_Task);
        }

        public IEditableProject EditableVersion
        {
            get { return new Project(this); }
        }

        internal Project()
            : base()
        {
            _Tasks = CreateTrackedCollection<IProjectTask>();
        }

        internal Project(Project P_Project)
            : base((LunaEntity)P_Project)
        {
            _Tasks = CreateTrackedCollection<IProjectTask>();
            BeginInit();
            LoadData(P_Project);

            foreach (var L_Task in P_Project.Tasks)
            {
                _Tasks.Add(new ProjectTask(L_Task));
            }

            EndInit();
        }

        public Project(IProjectData P_UserData)
            : base(P_UserData)
        {
            LoadData(P_UserData);
            AcceptChanges();
        }

        private void LoadData(IProjectData P_UserData)
        {
            _Name = P_UserData.Name;
            _Description = P_UserData.Description;
            _Code = P_UserData.Code;
            _Budget = P_UserData.Budget;
            _Start = P_UserData.Start;
            _End = P_UserData.End;
        }

        public override void BeginInit()
        {
            base.BeginInit();
            _Tasks.BeginInit();
        }

        public override void EndInit(bool P_IsStable = true)
        {
            _Tasks.EndInit();
            base.EndInit(P_IsStable);
        }
    }
}
