﻿using System;
using System.ComponentModel;
using Loki;

namespace Luna.UI.Project
{
    public abstract class ProjectGraph : LunaViewModel
    {
        public abstract object DataSource { get; }

        #region  StartDate
        private static PropertyChangedEventArgs _StartDateChangeArgs = ObservableHelper.CreateArgs<ProjectGraph>(x => x.StartDate);

        private DateTime _StartDate;

        public DateTime StartDate
        {
            get
            {
                return _StartDate;
            }

            set
            {
                if (value != _StartDate)
                {
                    _StartDate = value;
                    NotifyChanged(_StartDateChangeArgs);
                }
            }
        }
        #endregion

        #region  EndDate
        private static PropertyChangedEventArgs _EndDateChangeArgs = ObservableHelper.CreateArgs<ProjectGraph>(x => x.EndDate);

        private DateTime _EndDate;

        public DateTime EndDate
        {
            get
            {
                return _EndDate;
            }

            set
            {
                if (value != _EndDate)
                {
                    _EndDate = value;
                    NotifyChanged(_EndDateChangeArgs);
                }
            }
        }
        #endregion

        #region  ProjectID
        private static PropertyChangedEventArgs _ProjectIDChangeArgs = ObservableHelper.CreateArgs<ProjectGraph>(x => x.ProjectID);

        private Guid? _ProjectID;

        public Guid? ProjectID
        {
            get
            {
                return _ProjectID;
            }

            set
            {
                if (value != _ProjectID)
                {
                    _ProjectID = value;
                    NotifyChanged(_ProjectIDChangeArgs);
                }
            }
        }
        #endregion

        public ProjectGraph()
        {
            Init += AccountingLevelGraphViewModel_Init;
        }

        private void AccountingLevelGraphViewModel_Init(object P_Sender, EventArgs e)
        {
            this.WatchChange(this, x => x.StartDate, v => v.ReloadGraph);
            this.WatchChange(this, x => x.ProjectID, v => v.ReloadGraph);
            this.WatchChange(this, x => x.EndDate, v => v.ReloadGraph);
        }

        protected abstract void ReloadGraph(object P_Sender, EventArgs e);
    }
}
