﻿using System;
using System.Linq;
using Loki.Engine;
using Loki.Model;
using Luna.Data;
using Luna.Data.Project;

namespace Luna.Model.Project
{
    internal class CacheConfiguration : CacheConfiguration<LunaDataModel>
    {
        public static readonly ContainerConfiguration<IProject, Guid> PROJECTS = new ContainerConfiguration<IProject, Guid>()
        {
            LoadingErrorCode = ErrorCodes.PRO_010,
            FullLoader = () => DataContext.Project.GetAllProjects().Select(x => new Project(x)),
            Indexes = null,
            PrimaryKey = new UniqueKeyConfiguration<IProject, Guid>()
            {
                Indexer = u => u.ID,
                Loader = g => { IProjectData db = DataContext.Project.GetProject(g); return db == null ? null : new Project(db); }
            }
        };

        public static readonly ContainerConfiguration<ProjectTask, Guid> TASKS = new ContainerConfiguration<ProjectTask, Guid>()
        {
            LoadingErrorCode = ErrorCodes.PRO_011,
            FullLoader = () => DataContext.Project.GetAllTasks().Select(x => new ProjectTask(x)),
            Indexes = new IIndexConfiguration<ProjectTask>[]
            {
                new IndexConfiguration<ProjectTask, Guid>()
                {
                    Indexer = x => x.ProjectID,
                    Loader = x => DataContext.Project.GetAllTasksByProject((Guid)x).Select(t => new ProjectTask(t))
                }
            },
            PrimaryKey = new UniqueKeyConfiguration<ProjectTask, Guid>()
            {
                Indexer = u => u.ID,
                Loader = g => { IProjectTaskData db = DataContext.Project.GetTask(g); return db == null ? null : new ProjectTask(db); }
            }
        };
    }
}
